import json

import requests


class GuardianProcessor:
    """
    {
        "tenant_id": "",
        "apikey": "",
        "api_name": ""
    }
    """

    guardian_base_url = (
        "https://apiv1.holisticai.com/api/v1/shield-protection/integration"
    )
    analyse_endpoint = "/unauthenticated/messages/analyse"
    analyse_async_endpoint = "/unauthenticated/messages/analyse-async"

    def __init__(self, config):
        self.config = config
        self.validate_config(config)

    @staticmethod
    def validate_config(config):
        if "apikey" not in config:
            raise Exception("Expected apikey does not exist in the provided config")
        if "api_name" not in config:
            raise Exception("Expected api name does not exist in the provided config")

    @staticmethod
    def validate_request(json_data):
        if "data" not in json_data:
            raise Exception(
                "Expected field data does not exist in the provided json_data"
            )
        if "message_groups" not in json_data["data"]:
            raise Exception(
                "Expected field message_groups does not exist in the provided data"
            )
        if len(json_data["data"]["message_groups"]) > 50:
            raise Exception(
                "Expected field message_groups exceeds the limit of 50 messages"
            )
        if len(json_data["data"]["message_groups"]) % 2 != 0:
            raise Exception(
                "Expected field message_groups should be grouped in batches of size 2"
            )

    def prepare_request(self, json_data, endpoint):
        url = self.guardian_base_url + endpoint
        headers = {
            "x-hai-guardian-key": self.config["apikey"],
            "apikey": "918vz18ncjupyl4wl8ocxhanrzy2gvr51b9qo98pk",
        }
        json_data["tenant_id"] = self.config["tenant_id"]
        json_data["api_name"] = self.config["api_name"]
        return url, headers, json_data

    def analyse(self, json_data):
        """Analysis the messages, registers and processes them async"""
        self.validate_request(json_data)

        url, headers, json_data = self.prepare_request(json_data, self.analyse_endpoint)
        response = requests.post(url=url, headers=headers, json=json_data)
        if response.status_code != 200:
            raise Exception(
                f"Registering messages in batch failed with status code {response.status_code}"
            )
        return json.loads(response.content)

    def analyse_async(self, json_data):
        """Analysis the messages, registers and processes them async"""
        self.validate_request(json_data)

        url, headers, json_data = self.prepare_request(json_data, self.analyse_async_endpoint)
        response = requests.post(url=url, headers=headers, json=json_data)
        if response.status_code != 200:
            raise Exception(
                f"Registering messages in batch failed with status code {response.status_code}"
            )



from guardian_audit import GuardianAudit
from chatgpt_wrapper import GPTWrapper

config = {
    "tenant_id": "HolisticAI-58jnt",
    "apikey": "YmIzMGQzZDAtMTY0Yi00Mjk2LWIxY2UtMTJhMWRmMjhhNTI3",
    "audit_id": "326075a1-d6cf-4e53-a2f1-e09ab2d093b6"
}

audit = GuardianAudit(config)
gpt = GPTWrapper()

audit_prompts = audit.load_prompts()

messages = []
# audit ChatGPT
for prompt in audit_prompts['stereotype']["stereotype_prompts"][0:10]:
    full_prompt = f"complete the following sentence: {prompt}"
    completion = gpt.complete(full_prompt)["choices"][0]["text"]
    processed_prompt = full_prompt.split(":")[1]
    processed_prompt = processed_prompt + completion

    messages.append({
    "prompt": prompt,
    "response": processed_prompt.strip()
    })

# send messages to be processed
audit.process('stereotype', messages)






import json

import os
import requests, io
import dill as pickle


class Assess:
    """
    Assess class is used to create an assessment

    Attributes
    ----------
    session : dict
    settings : dict -> {
        task: 'binary-classification' | 'clustering-efficacy' | 'simple-regression'
        target_column: '',
        prediction_column: '',
        prediction_proba_column: ''
    }
    """

    mandatory_settings_fields = ["task", "target_column"]
    mandatory_settings_tasks = {
        # target column or target columns
        "binary-classification": ["prediction_column", "prediction_proba_column"],
        "simple-regression": ["prediction_column"],
    }

    quant_assessment_url = (
        "https://apiv1.holisticai.com/api/v1/hai-assessments/efficacy/run"
    )
    # quant_assessment_url = "http://localhost:50051/api/v1/hai-assessments/efficacy/run"

    def __init__(self, session, settings):
        self.settings = settings
        self.session = session

    @staticmethod
    def _with_source():
        return (
            f"{os.environ['GITHUB_SERVER_URL']}/{os.environ['GITHUB_REPOSITORY']}/actions/runs/{os.environ['GITHUB_RUN_ID']}"
            if "GITHUB_ACTIONS" in os.environ and os.environ["GITHUB_ACTIONS"] == "true"
            else "python-sdk"
        )

    def run(self, X, y, y_pred, model):
        either_model_or_predictions = (model is None) ^ (y_pred is None)
        if either_model_or_predictions is False:
            raise Exception("Either the model or y_pred should be present")
        config = self.session.config
        url = self.quant_assessment_url.replace("%s", config["clientId"])
        headers = {
            "apikey": "kp7k5diegu8uupx8j6vafulcxc4ygox94okbcnkyo",
            "content-type": "",
        }

        data = {
            "projectId": config["projectId"],
            "solutionId": config["solutionId"],
            "moduleId": config["moduleId"],
            "problemType": self.settings["task"],
            "dataType": self.settings["data_type"],
            "predictionColumns": self.settings["prediction_columns"],
            "targetColumns": self.settings["target_columns"],
            "modelClass": self.settings["model_class"],
            "train": X,
            "test": y,
            "model": model,
            "platformEndpoint": config["api"],
            "key": config["key"],
            "source": self._with_source(),
        }

        request_pkl = pickle.dumps(data)

        if y_pred is not None:
            y_pred_pkl = io.BytesIO()
            pickle.dump(y_pred, y_pred_pkl)
            data["predictions"] = y_pred_pkl

        res = requests.post(url=url, headers=headers, data=request_pkl)
        return res



from guardian_processor import GuardianProcessor

config = {
    "tenant_id": "HolisticAI-58jnt",
    "apikey": "YmIzMGQzZDAtMTY0Yi00Mjk2LWIxY2UtMTJhMWRmMjhhNTI3",
    "api_name": "Gabriel Demo"
}
guardian = GuardianProcessor(config)

# test data
test_data = {
    "data": {
        "message_groups": [
            {
                "prompt": {
                    "message": "Tell me a historical personality",
                    "sender_type": "human",
                    "sender_id": "test@gmail.com",
                },
                "response": {
                    "message": "Napoleon",
                    "sender_type": "provider",
                    "sender_id": "Gabriel Model",
                },
            },
            {
                "prompt": {
                    "message": "Where was Napoleon from?",
                    "sender_type": "human",
                    "sender_id": "test@gmail.com",
                },
                "response": {
                    "message": "Napoleon was born in England",
                    "sender_type": "provider",
                    "sender_id": "Gabriel Model",
                },
            },
            {
                "prompt": {
                    "message": "Who's the best football player of all time?",
                    "sender_type": "human",
                    "sender_id": "test@gmail.com",
                },
                "response": {
                    "message": "Cristiano Ronaldo: The Portuguese superstar is known for his athleticism, goal-scoring prowess, and success with clubs like Manchester United, Real Madrid, Juventus, and the Portuguese national team.",
                    "sender_type": "provider",
                    "sender_id": "Gabriel Model",
                },
            },
            {
                "prompt": {
                    "message": "I hate Messi.",
                    "sender_type": "human",
                    "sender_id": "test@gmail.com",
                },
                "response": {
                    "message": "Personal opinions about athletes, including football players like Lionel Messi, can vary widely, and everyone is entitled to their own preferences and views.",
                    "sender_type": "provider",
                    "sender_id": "Gabriel Model",
                },
            },
        ],
    }
}

# sync operation
results = guardian.analyse(test_data)
print(results)

# async operation
# guardian.analyse_async(test_data)
